@patmosx_level2_routines
@colorbar_local
@patmosx_standard_image_routines
@leapyr
@compute_month_day
@push
;-----------------------------------------------------------------------------
;$Id: patmosx_level2_view.pro,v 1.42 2015/01/23 20:30:16 heidinger Exp $
;-----------------------------------------------------------------------------
; An IDL routine to plot up a clavr-x/patmos-x level2 field
;
; mandatory
; file_name = name of the level2 file
; sds_index = the numerical index of the sds to plot
;
; optional
; sds_min = minimum value for the sds to plot
; sds_max = minimum value for the sds to plot
; coltab = color table index (0-47)
; cbartitle = color bar title
; path = path to hdf file
; rev = set to reverse color bar
; list = if set, programs shows a list of sds's in file
; clip = if set, values outside the range are noted
; magx = oversampling factor in x direction (to improve image)
; magy = oversampling factor in y direction (to improve image)
; xsize =  pixel size in x-direction
; ysize =  pixel size in x-direction
; box = lat_south, lon_west, lat_north, lon_east
; background = color index of background (0=black,1,2=grey,3=white,4+=color)
; projection = choose SATELLITE or MERCATOR
; nocbar = don not draw color bar, image spans whole window
; ncolors = ncolors
; storm
; domain = name of a predined box (usa-east, usa, usa-west)
;
; Issues - need to handle full orbits
;
;-------------------------------------------------------------------------------
pro patmosx_level2_view,file_name_input,sds_index, $
  SDS_MIN=sds_min, $
  SDS_MAX=sds_max, $
  COLTAB=coltab, $
  cbar_title=cbar_title, $
  nocbar=nocbar, $
  no_grid=no_grid, $
  path=path, $
  rev=rev, $
  list=list, $
  CLIP=clip, $
  cbar_divs=cbar_divs, $
  cont_color=cont_color, $
  fmt=fmt,  $
  magx=magx, $
  magy=magy, $
  xsize = xsize, $
  ysize = ysize, $
  box=box, $
  mask_cloud=mask_cloud, $
  qf_min=qf_min, $
  cloud_prob_max=cloud_prob_max, $
  cloud_prob_min=cloud_prob_min, $
  background=background, $
  projection=projection, $
  del_lon = del_lon, $
  del_lat = del_lat, $
  NCOLORS = ncolors, $
  mask_image=mask_image, $
  t11_image=t11_image, $
  cod_image=cod_image, $
  ceps_image=ceps_image, $
  cems_image=cems_image, $
  ctp_image=ctp_image, $
  cth_image=cth_image, $
  ctt_image=ctt_image, $
  type_image=type_image, $
  rain_image=rain_image, $
  sds_name = sds_name, $
  stats = stats, $
  storm = storm, $
  pmsl = pmsl, $
  time_stamp = time_stamp, $
  all = all, $
  multi = multi, $
  subsety = subsety, $
  subsetx = subsetx, $
  count_min = count_min, $
  png = png, $
  domain = domain, $
  goes = goes, $
  pixmap = pixmap, $
  hires_coast = hires_coast, $
  image_name = image_name, $
  missing_2 = missing_2


  ;-------- set defaults on optional keywords
  if (keyword_set(path) eq 0) then path = ''
  if (keyword_set(coltab) eq 0) then coltab = 0
  if (keyword_set(fmt) eq 0) then fmt = '(F6.2)'
  if (keyword_set(magx) eq 0) then magx = 1.0
  if (keyword_set(magy) eq 0) then magy = 1.0
  if (keyword_set(xsize) eq 0) then xsize = 600
  if (keyword_set(ysize) eq 0) then ysize = 600
  if (keyword_set(cont_color) eq 0) then cont_color = 0
  if (keyword_set(no_grid) eq 0) then no_grid = 0
  if (keyword_set(background) eq 0) then background=0
  if (keyword_set(projection) eq 0) then projection='MILLER_CYLINDRICAL'
  if (keyword_set(del_lon) eq 0) then del_lon = 5
  if (keyword_set(del_lat) eq 0) then del_lat = 5
  if (keyword_set(ncolors) eq 0) then ncolors = 250
  if (keyword_set(cloud_prob_max) eq 0) then cloud_prob_max = 1.0
  if (keyword_set(cloud_prob_min) eq 0) then cloud_prob_min = 0.0
  if (keyword_set(count_min) eq 0) then count_min = 100
  if (keyword_set(rev) eq 0) then begin
    irev = 0
  endif else begin
    irev = 1
  endelse
  it11 = keyword_set(t11_image)
  icod = keyword_set(cod_image)
  iceps = keyword_set(ceps_image)
  icems = keyword_set(cems_image)
  ictp = keyword_set(ctp_image)
  icth = keyword_set(cth_image)
  ictt = keyword_set(ctt_image)
  itype = keyword_set(type_image)
  imask = keyword_set(mask_image)
  irain = keyword_set(rain_image)

  missing = -999.0

  nfiles = 0
  if (keyword_set(all) ne 0 or keyword_set(multi) ne 0) then begin

    file_name_vector = file_search(path,'*'+file_name_input+'*.level2.hdf', count=nfiles)

  endif

  if (nfiles eq 0) then begin
    nfiles = 1
    file_name_vector = strarr(1)
    file_name_vector[0] = path+file_name_input
  endif

  for ifile = 0, nfiles -1 do begin

    ipos = strpos(file_name_vector[ifile],path)
    file_name = strmid(file_name_vector[ifile],ipos+strlen(path))

    ;--- construct full file name
    full_name = path + file_name

    ;--- open sd id
    sd_id = hdf_sd_start(full_name)

    ;--- read in some attributes
    hdf_sd_attrinfo, sd_id, hdf_sd_attrfind(sd_id,'WMO_SATELLITE_CODE'),data = wmo_sat_code
    hdf_sd_attrinfo, sd_id, hdf_sd_attrfind(sd_id,'START_YEAR'),data = start_year
    hdf_sd_attrinfo, sd_id, hdf_sd_attrfind(sd_id,'END_YEAR'),data = end_year
    hdf_sd_attrinfo, sd_id, hdf_sd_attrfind(sd_id,'START_TIME'),data = start_time
    hdf_sd_attrinfo, sd_id, hdf_sd_attrfind(sd_id,'START_DAY'),data = start_day
    hdf_sd_attrinfo, sd_id, hdf_sd_attrfind(sd_id,'END_TIME'),data = end_time
    hdf_sd_attrinfo, sd_id, hdf_sd_attrfind(sd_id,'END_DAY'),data = end_day

    mean_time = 0.5*(start_time + end_time)
    mean_hour = fix(mean_time)
    mean_minute = fix(60.0*(mean_time - mean_hour))

    ileap = leapyr(start_year)
    compute_month_day, start_day, ileap, month, day_of_month

    case wmo_sat_code of

      3:   sat_name = "metop-01"
      4:   sat_name = "metop-02"
      5:   sat_name = "metop-03"
      171: sat_name = "mtsat-1r"
      172: sat_name = "mtsat-02"
      200: sat_name = "noaa-08"
      201: sat_name = "noaa-09"
      202: sat_name = "noaa-10"
      203: sat_name = "noaa-11"
      204: sat_name = "noaa-12"
      204: sat_name = "noaa-12"
      205: sat_name = "noaa-14"
      206: sat_name = "noaa-15"
      207: sat_name = "noaa-16"
      208: sat_name = "noaa-17"
      209: sat_name = "noaa-18"
      223: sat_name = "noaa-19"
      224: sat_name = "npp"
      257: sat_name = "goes-13"
      258: sat_name = "goes-14"
      259: sat_name = "goes-15"
      706: sat_name = "noaa-06"
      707: sat_name = "noaa-07"
      708: sat_name = "noaa-05"
      783: sat_name = "terra-modis"
      784: sat_name = "aqua-modis"

    endcase

    ;----- if option selected to list sds's, do so
    if (keyword_set(list) ne 0) then begin
      hdf_sd_fileinfo, sd_id, num_sds,num_attr
      for isds = 0, num_sds-1 do begin
        sds_id = hdf_sd_select(sd_id, isds)
        hdf_sd_getinfo, sds_id, name=sds_name_list
        print, isds, ': ', sds_name_list
      endfor
      hdf_sd_end,sd_id
      stop
    endif

    ;-- get sds_name from sds_index
    if (keyword_set(sds_name) ne 0) then begin
      sds_index = hdf_sd_nametoindex(sd_id,sds_name)
      if (sds_index lt 0) then begin
        print, sds_name, ' does not exist in this file'
        return
      endif
    endif else begin
      ;-- get sds_name from sds_index
      sds_name = ' '
      sds_id = hdf_sd_select(sd_id, sds_index)
      hdf_sd_getinfo, sds_id, name=sds_name
    endelse


    ;--- check if quality flags need to be read in
    if (keyword_set(qf_min) ne 0) then begin
      if (sds_name eq 'cld_opd_dcomp') then begin
        qf_name = 'cld_opd_dcomp_qf'
        qf_index = hdf_sd_nametoindex(sd_id,qf_name)
      endif
      if (sds_name eq 'cld_reff_dcomp') then begin
        ;      qf_name = 'cld_reff_dcomp_qf'
        ;      qf_index = hdf_sd_nametoindex(sd_id,qf_name)
        qf_name = 'dcomp_quality'
        qf_index = hdf_sd_nametoindex(sd_id,qf_name)
      endif
      if (sds_name eq 'cld_temp_acha') then begin
        qf_name = 'cld_temp_acha_qf'
        qf_index = hdf_sd_nametoindex(sd_id,qf_name)
      endif
      if (sds_name eq 'cld_emiss_acha') then begin
        qf_name = 'cld_emiss_acha_qf'
        qf_index = hdf_sd_nametoindex(sd_id,qf_name)
      endif
      if (sds_name eq 'cld_beta_acha') then begin
        qf_name = 'cld_beta_acha_qf'
        qf_index = hdf_sd_nametoindex(sd_id,qf_name)
      endif
      if (sds_name eq 'aot_0_65um_nom') then begin
        qf_name = 'aot_qf'
        qf_index = hdf_sd_nametoindex(sd_id,qf_name)
      endif
      if (qf_index gt 0) then begin
        qf_temp = read_standard_hdf(full_name,qf_name)
        if (keyword_set(multi) ne 0) then begin
          push, qf, qf_temp  & qf_temp = 0
        endif else begin
          qf = qf_temp & qf_temp = 0
        endelse
        push, qf, qf_temp  & qf_temp = 0
        qf_read_flag = 1
      endif else begin
        qf_read_flag = 0
      endelse
    endif else begin
      qf_read_flag = 0
    endelse
    ;--- close
    hdf_sd_end, sd_id

    ;--- read dcomp qf
    if (sds_name eq 'cld_reff_dcomp') then begin
      dcomp_qf = read_standard_hdf(full_name,'dcomp_quality')
      dcomp_info = read_standard_hdf(full_name,'dcomp_info')

      UNPACK_DCOMP_QUALITY_FLAGS, dcomp_qf, DCOMP_Processed_Flag,  $
        COD_Valid_Qf, REF_Valid_Qf, $
        COD_Degraded_Qf, REF_Degraded_Qf,  $
        DCOMP_Converged_Qf, DCOMP_Glint_Qf
    endif

    ;--- compute lat, lon for each pixel
    scan_time_temp = read_standard_hdf(full_name,'scan_line_time', /quiet)
    if (keyword_set(multi) ne 0) then begin
      push, scan_time, scan_time_temp & scan_time_temp = 0
    endif else begin
      scan_time = scan_time_temp & scan_time_temp = 0
    endelse

    lat_temp = read_standard_hdf(full_name,'latitude', /quiet)
    if (keyword_set(multi) ne 0) then begin
      push, lat, lat_temp & lat_temp = 0
    endif else begin
      lat = lat_temp & lat_temp = 0
    endelse

    lon_temp = read_standard_hdf(full_name,'longitude', /quiet)
    if (keyword_set(multi) ne 0) then begin
      push, lon, lon_temp & lon_temp = 0
    endif else begin
      lon = lon_temp & lon_temp = 0
    endelse

    ;--- determine number of pixels per scan (GAC=409, HRPT=2048)
    dims = size(lat)
    num_elems = dims[1]
    num_lines = dims[2]

    if (keyword_set(mask_cloud) ne 0) then begin
      cloud_mask_temp = read_standard_hdf(full_name,'cloud_mask')
      if (keyword_set(multi) ne 0) then begin
        push, cloud_mask, cloud_mask_temp & cloud_mask_temp = 0
      endif else begin
        cloud_mask = cloud_mask_temp & cloud_mask_temp = 0
      endelse
    endif
    cloud_mask_read_flag = 0
    if (n_elements(cloud_mask) gt 1) then cloud_mask_read_flag = 1


    cloud_probability = -1
    if (cloud_prob_max ne 1 or cloud_prob_min ne 0) then begin
      cloud_probability_temp = read_standard_hdf(full_name,'cloud_probability')
      if (keyword_set(multi) ne 0) then begin
        push, cloud_probability, cloud_probability_temp & cloud_probability_temp = 0
      endif else begin
        cloud_probability = cloud_probability_temp & cloud_probability_temp = 0
      endelse
    endif
    cloud_prob_read_flag = 0
    if (n_elements(cloud_probability) gt 1) then begin
      cloud_prob_read_flag = 1
    endif

    ;--- knowing, index number, read data
    z_temp = float(read_standard_hdf(full_name,sds_name))
    if (keyword_set(multi) ne 0) then begin
      push, z, z_temp & z_temp = 0
    endif else begin
      z = z_temp & z_temp = 0
    endelse

    if (sds_name eq 'cloud_mask' or sds_name eq 'cloud_mask_1b') then begin
      land_class_temp = read_standard_hdf(full_name,'land_class')
      if (keyword_set(multi) ne 0) then begin
        push, land_class, land_class_temp & land_class_temp = 0
      endif else begin
        land_class = land_class_temp & land_class_temp = 0
      endelse
    endif

    land_class_read_flag = 0
    if (n_elements(land_class) gt 1) then begin
      land_class_read_flag = 1
    endif


    if (keyword_set(multi) ne 0 and ifile lt nfiles -1) then begin
      goto, skip_file
    endif

    ;---------------------------------------------------------------------------
    ; Start imaging
    ;---------------------------------------------------------------------------



    ;--- set low quality to missing
    if (qf_read_flag eq 1) then begin
      index_invalid = where(qf lt qf_min, count_invalid)
      if (count_invalid gt 0) then z[index_invalid] = missing
    endif

    ;--- apply cloud prob filter
    if (cloud_prob_read_flag eq 1) then begin
      print, 'apply filter'
      index_invalid = where(cloud_probability ne missing and $
        (cloud_probability lt cloud_prob_min or $
        cloud_probability gt cloud_prob_max), count_invalid)
      if (count_invalid gt 0) then z[index_invalid] = missing
    endif

    ;--- if min and max range not in command line input, determine it
    index = where(z ne missing, count_valid)
    if (count_valid eq 0) then begin
      print, "Error: no valid data, returning"
      ;   return
    endif
    if (keyword_set(sds_min) eq 0) then begin
      if (count_valid gt 0) then begin
        sds_min = min(z[index])
      endif
    endif
    if (keyword_set(sds_max) eq 0) then begin
      if (count_valid gt 0) then begin
        sds_max = max(z[index])
      endif
    endif
    if (keyword_set(missing_2) ne 0) then begin
      if (missing_2 eq -101) then missing_2 = 0.0
      index_missing_2 = where(z eq 0.0, count_missing_2)
      if (count_missing_2 gt 0) then z[index_missing_2] = missing
    endif
    ;ii = where(z lt 0.01)
    ;z[ii] = missing
    if (keyword_set(cbar_divs) eq 0) then begin
      cbar_divs = 5
    endif
    if (sds_min eq -101) then begin
      sds_min = 0.0
    endif

    ;---- set cbar title to sds_name if not set yet
    if (keyword_set(cbar_title) eq 0) then cbar_title = sds_name

    ;--- create window
    if (keyword_set(pixmap) ne 0) then begin
      window, xsize=xsize,ysize=ysize, /free, /pixmap
    endif else begin
      window, xsize=xsize,ysize=ysize, /free
    endelse

    ;--- load color table (13 = rainbow)
    load_color_table, coltab, irev, 1.0, ncolors
    erase, color = 3
    blank_color = 3
    missing_color = 2
    missing_color_map = background
    ;missing_color_map = 1

    ;---- set colors for values the exceed the plotted range
    under_range_index = 4
    over_range_index = 5
    ;under_range = REFORM([100,100,100],1,3)
    ;over_range = REFORM([55,255,255],1,3)
    over_range = REFORM([50,50,50],1,3)
    under_range = REFORM([50,50,50],1,3)
    tvlct, under_range, under_range_index
    tvlct, over_range, over_range_index


    ;--- convert to an image
    minz= sds_min
    maxz= sds_max
    ispecial = it11 + icod + iceps + ictp + icth + ictt + itype + imask + irain + icems

    if (ispecial eq 0) then begin
      image = bytscl(z,min=minz,max=maxz, top=!d.table_size-7) + 6B
      index = where(z eq missing, count)
      if (count gt 0) then image[index] = missing_color

      if (keyword_set(clip) ne 0) then begin
        index = where(z lt minz and z ne missing, count)
        if (count gt 0) then image[index] = under_range_index
        index = where(z gt maxz and z ne missing, count)
        if (count gt 0) then image[index] = over_range_index
      endif

      if (cloud_mask_read_flag eq 1) then begin
        index = where(cloud_mask eq 3, count)
        if (count gt 0) then image[index] = 3
        index = where(cloud_mask eq 2, count)
        if (count gt 0) then image[index] = 3
      endif

    endif else begin


      if (icems ne 0) then image = cloud_emissivity_image(z)
      if (it11 ne 0) then image = t11_image(z)
      if (icod ne 0) then image = cloud_optical_depth_image(z)
      if (iceps ne 0) then image = cloud_effective_radius_image(z)
      if (ictp ne 0) then image = cloud_pressure_image(z)
      if (icth ne 0) then image = cloud_height_image(z)
      if (ictt ne 0) then image = cloud_temperature_image(z)
      if (itype ne 0) then image = cloud_type_image(z)
      if (imask ne 0) then image = cloud_mask_image(z,land_class)
      if (irain ne 0) then image = rain_rate_image(z)

    endelse

    ;--- resize image and remove space
    dims = size(z)
    nx = dims[1]
    ny = dims[2]


    ;----------------------------------
    ;--- determine map area
    ;----------------------------------
    if (keyword_set(domain) ne 0) then begin

      case domain OF
        'conus-east': box = [20,-112,55,-62]
        'conus-west': box = [15,-160,55,-100]
        'great-lakes': box = [35,-95,55,-75]
        'taiwan': box = [15,110,35,130]
        'mtsat': box = [-70,75,70,-165]
        'coms': box = [-70,70,70,-170]
        'arctic': box = [70,-180,90,180]
        else: begin
          print, 'domain not recognized'
          domain = "no-grid"
        end
      endcase
    endif else begin
      domain = "no-grid"
    endelse

    if (keyword_set(box) ne 0 or domain ne 'no-grid') then begin

      if (keyword_set(box) ne 0) then begin
        if (n_elements(box) ne 4) then begin
          print, "box elements should be lat_south, lon_west, lat_north and lon_east"
          stop
        endif
      endif

      map_limit = box
      lat_south_sub = box[0]
      lon_west_sub = box[1]
      lat_north_sub = box[2]
      lon_east_sub = box[3]
    endif  else begin
      index = where(lat ne -90.0 and lat ne missing and lon ne missing)
      lat_south_sub = min(lat[index])
      lat_north_sub = max(lat[index])
      lon_west_sub = min(lon[index])
      lon_east_sub = max(lon[index])
    endelse

    print, 'after dateline ', lon_west_sub, lon_east_sub

    ;--- dateline processing
    date_line = 0
    if (lon_east_sub lt lon_west_sub) then begin
      date_line = 1
    endif

    if (date_line eq 1) then begin
      temp = lon_east_sub
      lon_east_sub = temp + 360.0
      idx = where(lon lt 0.0,cc)
      lon[idx] = lon[idx] + 360
    endif

    print, 'after dateline ', lon_west_sub, lon_east_sub


    ;--- make image name
    if (keyword_set(image_name) eq 0) then begin
      image_name  = 'patmosx.'+sat_name+'.'+sds_name + '.' + domain + '.'+$
        string(start_year,format="(I04)")+string(month,format="(I02)")+string(day_of_month,format="(I02)")+ $
        "."+string(mean_hour,format="(I02)")+string(mean_minute,format="(I02)")+'00.png'
    endif


    ;----------------------------------------------------------
    ; remove data outside box if subsetted
    ;----------------------------------------------------------
    if (keyword_set(subsety)) then begin
      y = reform(lat[nx/2,*])
      ;  x = reform(lon[nx/2,*])
      index_geoy = where(y gt lat_south_sub - 2.0 and y lt lat_north_sub + 2.0, count_geoy)
      lat = lat[*,index_geoy]
      lon = lon[*,index_geoy]
      if (size(cloud_mask,/TYPE) ne 0) then cloud_mask = cloud_mask[*,index_geoy]
      if (size(cloud_probability,/TYPE) ne 0) then cloud_probability = cloud_probability[*,index_geoy]
      image = image[*,index_geoy]
      if (size(land_class,/TYPE) ne 0) then land_class = land_class[*,index_geoy]
      z = z[*,index_geoy]
      if (count_geoy le count_min) then begin
        print, 'Insufficient Data, skipping file'
        if (keyword_set(png) ne 0) then begin
          wdelete
        endif
        goto, skip_file
      endif
    endif
    ;--- resize image and remove space
    dims = size(z)
    nx = dims[1]
    ny = dims[2]
    if (keyword_set(subsetx)) then begin
      x = reform(lon[*,ny/2])
      index_geox = where(x gt lon_west_sub - 2.0 and y gt lon_east_sub + 2.0, count_geox)
      lat = lat[index_geox,*]
      lon = lon[index_geox,*]
      if (size(cloud_mask,/TYPE) ne 0) then cloud_mask = cloud_mask[index_geox,*]
      if (size(cloud_probability,/TYPE) ne 0) then cloud_probability = cloud_probability[index_geox,*]
      image = image[index_geox,*]
      if (size(land_class,/TYPE) ne 0) then land_class = land_class[index_geox,*]
      z = z[index_geox,*]
      if (count_geox le count_min) then begin
        print, 'Insufficient Data, skipping file'
        if (keyword_set(png) ne 0) then begin
          wdelete
        endif
        goto, skip_file
      endif
    endif

    ;--- resize image and remove space
    dims = size(z)
    nx = dims[1]
    ny = dims[2]


    ;----------------------------------------------------------
    ; move box to be centered on storm
    ;----------------------------------------------------------
    if (keyword_set(storm)) then begin

      pmsl_temp = read_standard_hdf(full_name,'mean_sealevel_pressure_background', /quiet)
      push, pmsl, pmsl_temp  & pmsl_temp = 0
      pmsl = pmsl[*,index_geoy]
      index_spatial = where(lat gt lat_south_sub and lat lt lat_north_sub and lon gt lon_west_sub and lon lt lon_east_sub, $
        count_spatial)
      pmsl_min = min(pmsl[index_spatial],location)
      ind = ARRAY_INDICES(pmsl[index_spatial],location)
      lat_center_sub = lat[index_spatial[ind[0]]]
      lon_center_sub = lon[index_spatial[ind[0]]]

      box_width = box[3] - box[1]
      box_height = box[2] - box[0]

      lat_south_sub = lat_center_sub - box_height/2.0
      lat_north_sub = lat_center_sub + box_height/2.0
      lon_west_sub = lon_center_sub - box_height/2.0
      lon_east_sub = lon_center_sub + box_height/2.0
    endif

    ;------------------------------------------------------------------
    ; set mapping parameters
    ;------------------------------------------------------------------
    map_limit = [lat_south_sub, lon_west_sub, lat_north_sub, lon_east_sub]
    lat_center = 0.0
    lon_center = 0.5*(lon_west_sub + lon_east_sub)
    if (lon_west_sub gt lon_east_sub) then begin
      lon_center = 0.5*(lon_west_sub + 360.0 + lon_east_sub)
    endif
    if (lon_center gt 180.0) then lon_center = lon_center - 360.0

    ;--- expand image to fill holes
    image = congrid(image,magx*nx,magy*ny)
    lat = congrid(lat,magx*nx,magy*ny,/interp)
    lon = congrid(lon,magx*nx,magy*ny,/interp)
    index = where(lat lt 90 and lat gt -90)

    ;--- set up map
    erase, color = 3

    Image_Position = [0.01,0.15,0.99,0.95]
    Cbar_Position = [0.15,0.05,0.85,0.08]
    if (keyword_set(nocbar) ne 0) then Image_Position = [0.001,0.001,0.999,0.999]

    if (projection eq 'LCC') then begin
      MAP_SET,/CONIC,27.25,128.5,STANDARD_PARALLELS=[30,60],LIMIT=[-1,102,40,80.1,48,155,2.4,142],/NOBORDER,/NOERASE, $
        Position = Image_Position, /HORIZON
    endif
    if (projection eq 'MERCATOR') then begin
      map_set, lat_center, lon_center, /MERCATOR, $
        limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
        Position = Image_Position, /HORIZON, /noerase
    endif
    if (projection eq 'MOLLWEIDE') then begin
      map_set, lat_center, lon_center, /MOLLWEIDE, $
        limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
        Position = Image_Position, /HORIZON, /noerase
    endif
    if (projection eq 'MILLER_CYLINDRICAL') then begin
      map_set, lat_center, lon_center, /MILLER_CYLINDRICAL, $
        limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
        Position = Image_Position, /HORIZON, /noerase
      print, 'called map_set with ', [lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub]
    endif
    if (projection eq 'ARCTIC') then begin
      map_set, 90.0, 0.0, /STEREOGRAPHIC, $
        limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
        Position = Image_Position, /HORIZON, /noerase
    endif
    if (projection eq 'SATELLITE') then begin
      if (keyword_set(box) eq 0) then begin
        lat_center = lat[nx/2,ny/2]
        lon_center = lon[nx/2,ny/2]
      endif else begin
        lat_center = 0.5*(lat_south_sub + lat_north_sub)
        lon_center = 0.5*(lon_west_sub + lon_east_sub)
      endelse
      map_set, lat_center, lon_center, /SATELLITE,sat_p=[1.10,0,0], $
        Position = Image_Position, /HORIZON, /noerase
    endif
    if (projection eq 'GOES') then begin
      lat_center = 0.5*(lat_south_sub + lat_north_sub)
      lon_center = 0.5*(lon_west_sub + lon_east_sub)
      map_set, lat_center, lon_center, /SATELLITE,sat_p=[6.10,0,0], $
        Position = Image_Position, /HORIZON, /noerase
    endif
    if (projection eq 'HAMMER') then begin
      map_set, lat_center, lon_center, /HAMMER, $
        limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
        Position = Image_Position, /HORIZON, /noerase
    endif
    if (projection eq 'STEREOGRAPHIC') then begin
      map_set, 90.0, 0.0, /STEREOGRAPHIC, $
        limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
        Position = Image_Position, /HORIZON, /noerase
    endif

    if (projection eq 'OPTIMAL') then begin
      lat_center = lat[nx/2,ny/2]
      lon_center = lon[nx/2,ny/2]
      print, 'lat center = ', lat_center
      if (lat_center gt 70) then begin
        map_set, lat_center, lon_center, /STEREOGRAPHIC, $
          limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
          Position = Image_Position, /HORIZON, /noerase
      endif
      if (lat_center lt -70) then begin
        map_set, -90.0, 0.0, /STEREOGRAPHIC, $
          limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
          Position = Image_Position, /HORIZON, /noerase
      endif
      if (lat_center ge -70 and lat_center le 70) then begin
        map_set, lat_center, lon_center, /MILLER_CYLINDRICAL, $
          limit=[lat_south_sub,lon_west_sub,lat_north_sub,lon_east_sub], $
          Position = Image_Position, /HORIZON, /noerase
      endif
    endif

    ;erase, color = 3
    ;--- use liam's imagemap
    imagemap, image[index],lat[index],lon[index], $
      /noborder, /current, /noerase, $
      Range = [0,!d.table_size-1],missing=missing_color_map

    ; annontate
    if (no_grid eq 0) then begin
      if (keyword_set(hires_coast)) then begin
        map_continents, /hires, /Coasts, Color = cont_color,thick=1, /USA
      endif else begin
        map_continents, /lores, /Coasts, Color = cont_color,thick=1, /USA
      endelse
      map_grid,latdel = del_lat, londel = del_lon, label=1, Color=cont_color,Size=1.0
    endif

    ;--- apply fanning's color bar routine
    csize = 1.5
    if (keyword_set(nocbar) eq 0 and ispecial eq 0) then begin
      colorbar_local, maxrange=maxz,minrange=minz, bottom = 6, title = cbar_title, $
        position = Cbar_Position, color = 0, charsize=csize,format=fmt, $
        divisions = cbar_divs
    endif

    if (keyword_set(clip) ne 0) then begin
      polyfill, [0.02,0.05,0.05,0.02], [0.05,0.05,0.08,0.08], color = under_range_index, /normal
      plots, [0.02,0.05,0.05,0.02, 0.02], [0.05,0.05,0.08,0.08,0.05], color = 0, /normal
      xyouts, 0.02, 0.02, '<', size = csize, color = 0, /normal

      polyfill, [0.96,0.99,0.99,0.96], [0.05,0.05,0.08,0.08], color = over_range_index, /normal
      plots, [0.96,0.99,0.99,0.96,0.96], [0.05,0.05,0.08,0.08,0.05], color = 0, /normal
      xyouts, 0.97, 0.02, '>', size = csize, color = 0, /normal
    endif

    if (keyword_set(nocbar) eq 0) then begin
      if (it11 eq 1) then t11_color_bar
      if (icod eq 1) then cloud_optical_depth_color_bar
      if (iceps eq 1) then cloud_effective_radius_color_bar
      if (ictp eq 1) then cloud_pressure_color_bar
      if (icth eq 1) then cloud_height_color_bar
      if (icems eq 1) then cloud_emissivity_color_bar
      if (ictt eq 1) then cloud_temperature_color_bar
      if (itype eq 1) then cloud_type_color_bar
      if (irain eq 1) then rain_rate_color_bar
      if (imask eq 1) then cloud_mask_color_bar, land_class_read_flag
    endif


    if (keyword_set(nocbar) eq 0) then begin
      thestring = strtrim(file_name,2)
      if (keyword_set(time_stamp) ne 0) then begin
        thestring = sds_name + "_" + time_string
      endif
      if (strlen(thestring) ge 10) then xyouts, 0.5, 0.96, ALIGN=0.5, thestring, color=0, CHARSIZE=STR_SIZE(thestring, 0.4), /normal
      if (strlen(thestring) lt 10) then xyouts, 0.5, 0.96, ALIGN=0.5, thestring, color=0, CHARSIZE=STR_SIZE(thestring, 0.2), /normal
      thestring = 'AIT Delivery 12/14'
      ;thestring = 'Central Weather Bureau'
      xyouts, 0.12, 0.98, ALIGN=0.5, thestring, color=2, CHARSIZE=STR_SIZE(thestring, 0.2), /normal
    endif

    if (keyword_set(png) ne 0) then begin
      saveimage, image_name, /png
      wdelete
    endif

    ;---------------------------------------------------------------------
    ; mean
    ;---------------------------------------------------------------------
    if (keyword_set(stats) ne 0) then begin
      mean_value = missing
      index = where(z ne missing, count_valid)
      weighted_sum = total(z[index]*cos(lat[index]*!dtor))
      weight = total(cos(lat[index]*!dtor))
      mean_value = weighted_sum / weight
      print, "MEAN OF VALID DATA = ", Mean_Value
    endif

    skip_file:

  endfor

end
